/***********************************************************
 *
 * d_SDRAM_FLASH.h
 * SYSTART GmbH
 * SAM, 23.06.2017
 *
 ***********************************************************/

#ifndef __D_SDRAM_FLASH_H_
#define __D_SDRAM_FLASH_H_

/***********************************************************
 * Includes
 ***********************************************************/

#include "stm32f4xx.h"
#include "global_Memory.h"

/***********************************************************
 * Function declaration
 ***********************************************************/
/**
* @brief Initialization of external SDRAM and NOR-Flash
*
* -
*
* @param -
*
* @return -
*
*/
void SDRAM_FLASH_init ( void );

/**
* @brief Writes 8bit data to the specified SDRAM memory address
*
* -
*
* @param adr 		Address
* @param data 		8bit data to be written
*
* @return -
*
*/
void SDRAM_Write_8b ( uint32_t adr, uint8_t data );

/**
* @brief Writes 16bit data to the specified SDRAM memory address
*
* -
*
* @param adr 		Address
* @param data 		16bit data to be written
*
* @return -
*
*/
void SDRAM_Write_16b ( uint32_t adr, uint16_t data );

/**
* @brief Writes 32bit data to the specified SDRAM memory address
*
* -
*
* @param adr 		Address
* @param data 		32bit data to be written
*
* @return -
*
*/
void SDRAM_Write_32b ( uint32_t adr, uint32_t data );

/**
* @brief Reads 8bit data from the specified SDRAM memory address
*
* -
*
* @param adr 		Address
*
* @return uint8_t	Read data
*
*/
uint8_t SDRAM_Read_8b ( uint32_t adr );

/**
* @brief Reads 16bit data from the specified SDRAM memory address
*
* -
*
* @param adr 		Address
*
* @return uint16_t	Read data
*
*/
uint16_t SDRAM_Read_16b ( uint32_t adr );

/**
* @brief Reads 32bit data from the specified SDRAM memory address
*
* -
*
* @param adr 		Address
*
* @return uint32_t	Read data
*
*/
uint32_t SDRAM_Read_32b ( uint32_t adr );

#endif // __D_SDRAM_H_
