/***********************************************************
 *
 * global_Memory.h
 * SYSTART GmbH
 * SAM, 24.11.2016
 *
 ***********************************************************/

#ifndef __GLOBAL_MEMORY_H_
#define __GLOBAL_MEMORY_H_

/***********************************************************
 * Includes
 ***********************************************************/
#include "stm32f4xx.h"


/***********************************************************
 * Defines
 ***********************************************************/

// ***************    Defines SDRAM   *****************
#define SDRAM_MODEREG_BURST_LENGTH_1             	((uint16_t)0x0000)
#define SDRAM_MODEREG_BURST_LENGTH_2             	((uint16_t)0x0001)
#define SDRAM_MODEREG_BURST_LENGTH_4             	((uint16_t)0x0002)
#define SDRAM_MODEREG_BURST_LENGTH_8             	((uint16_t)0x0004)
#define SDRAM_MODEREG_BURST_TYPE_SEQUENTIAL      	((uint16_t)0x0000)
#define SDRAM_MODEREG_BURST_TYPE_INTERLEAVED     	((uint16_t)0x0008)
#define SDRAM_MODEREG_CAS_LATENCY_2              	((uint16_t)0x0020)
#define SDRAM_MODEREG_CAS_LATENCY_3              	((uint16_t)0x0030)
#define SDRAM_MODEREG_OPERATING_MODE_STANDARD    	((uint16_t)0x0000)
#define SDRAM_MODEREG_WRITEBURST_MODE_PROGRAMMED 	((uint16_t)0x0000)
#define SDRAM_MODEREG_WRITEBURST_MODE_SINGLE     	((uint16_t)0x0200)
#define SDRAM_REFRESH_COUNT						 	((uint32_t)0x0603)

#define SDRAM_START_ADDR							((uint32_t)0xC0000000)	//3221225472


// ***************    Defines NOR-Flash   *****************
#define BlockErase_Timeout    ((uint32_t)0x00A00000)
#define ChipErase_Timeout     ((uint32_t)0x30000000)
#define Program_Timeout       ((uint32_t)0x00002800) //((uint32_t)0x00001400)

#define NOR_FLASH_START_ADDR       ((uint32_t)0x60000000)

typedef struct
{
	uint16_t Manufacturer_Code;
	uint16_t Device_Code1;
	uint16_t Device_Code2;
	uint16_t Device_Code3;
}NOR_IDTypeDef;

typedef enum
{
	NOR_SUCCESS = 0,
	NOR_ONGOING,
	NOR_ERROR,
	NOR_TIMEOUT
}NOR_Status;

// Define fuer Speicherflag
#define FLASH_MEMFLAG										0xA5A5

// ***************    Defines EEPROM M95512W    *****************
// Defines fuer die Pages
// Page Size 128byte, 512 Pages
#define EEPROM_START_ADDRESS		((uint16_t)0x0000)
#define EEPROM_PAGE_SIZE			((uint16_t)0x0080)
#define EEPROM_PAGE_COUNT			((uint16_t)0x0200)

// Defines fuer Instructions
#define WREN						((uint8_t)0x06)
#define WRDI						((uint8_t)0x04)
#define RDSR						((uint8_t)0x05)
#define WRSR						((uint8_t)0x01)
#define READ						((uint8_t)0x03)
#define WRITE						((uint8_t)0x02)

// Define fuer Speicherflag
#define EEPROM_MEMFLAG										0xA5


typedef struct
{
	uint8_t ui8_page;
	uint8_t ui8_zelle;
}EEPROM_Adresse_t;

/***********************************************************
 * Globale Variablen
 ***********************************************************/
extern uint32_t	error_count_eeprom;

#endif // __GLOBAL_MEMORY_H_
