/*****************************************************************
 *  File: touch.c
 *
 *  Summary: Read touch coordinates
 *
 ****************************************************************/

/*****************************************************************
 * Includes
 ****************************************************************/
#include <stdbool.h>
#include <application/touch.h>
#include "global_Display_Touch.h"
#include "driver/d_Touch_FT5316.h"
#include "driver/d_Display_DMA2D.h"
#include "application/uart_rs232_comm.h"

extern tFont courier_new;

/*****************************************************************
 *
 * Function:  void process_touch_input(void)
 *
 * Summary:   Read touch input and populate the touch response struct
 * Parameters:
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
void process_touch_input(void) {
	static bool state = RELEASE;
	TouchXY_t txy;
	static int touch_snd = 0;
	static tTimer timeout;

	if (touch_snd) {
		if(timer_get_time_ms() - timeout > 500)
			return;
		touch_snd = 0;
	}

	txy = Touch_FT5316_getTouchXY();
	// if touch event
	if (txy.event == TOUCH) {
		if (state != TOUCH) {
			state = TOUCH;
			send_touch_response(&txy);
			touch_snd = 1;
			//Area check
			if(AreaCheck(0, 0, HDP/2, VDP/2)){
				DMA2D_Fill_Color(0xFF00FF00, Layer_1, Buffer_1);
			}
			else{
				DMA2D_Fill_Color(0xFFFF0000, Layer_1, Buffer_1);
			}
			timeout = timer_get_time_ms();
		}
	}
	//else if release event
	else if (txy.event == RELEASE) {
		if (state != RELEASE) {
			state = RELEASE;
			send_touch_response(&txy);
			touch_snd = 1;
			timeout = timer_get_time_ms();
			DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);
			DMA2D_write_string("Touch Me!", (HDP-9*16)/2, VDP/2-11, 0xFF000000, &courier_new, Layer_1, Buffer_1, Layer_1, Buffer_1);
		}
	}
	//else if timeout error in i2c communication
	else if (txy.event == 0xFF) {
		send_touch_response(&txy);
		timeout = timer_get_time_ms();
	}
}
