/*****************************************************************
 *  File: uart_rs232_comm.c
 *
 *  Summary: Functions to send, receive and parse uart data
 *
 ****************************************************************/
 
/***********************************************************
 * Includes
 ***********************************************************/
#include "application/uart_rs232_comm.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


/***********************************************************
 * Public Functions
 ***********************************************************/

/*****************************************************************
 *
 * Function: comm_err_t send_touch_response(touch_resp_t *touch_resp)
 *
 * Summary: Pack the touch coordinates into uart command format and
 * 			send the command over uart
 *
 * Parameters:
 *     touch_rep - pointer to the struct holding touch coordinates
 *     			   and event type
 *
 * Return:
 *  0 - if success
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
comm_err_t send_touch_response(TouchXY_t *touch_resp) {
	char output[49];
	tTimer send_timeout;

	if(touch_resp->event == 0x00){
		sprintf(output, "X,Y-Coordinates: (%3i,%3i) EVENT-TYPE:   TOUCH\r\n", touch_resp->xPos, touch_resp->yPos);
	}
	else if (touch_resp->event == 0x01){
		sprintf(output, "X,Y-Coordinates: (%3i,%3i) EVENT-TYPE: RELEASE\r\n", touch_resp->xPos, touch_resp->yPos);
	}
	else if (touch_resp->event == 0xFF){
		sprintf(output, "X,Y-Coordinates: (---,---) EVENT-TYPE:   ERROR\r\n");
	}


	send_timeout = timer_get_time_ms();
	uart_SendString(COM1, output, 0);
	if(timer_get_time_ms() - send_timeout > 10) return FCN_TIMEOUT;


	return FCN_OK;
}
