/***********************************************************
 *
 * d_Display_DMA2D.c
 * SYSTART GmbH
 * JOH, 27.11.2018
 *
 ***********************************************************/

 /***********************************************************
 * Includes
 ***********************************************************/
#include "driver/d_Display_DMA2D.h"
#include <stdlib.h>

#ifndef _swap_int16_t
	#define _swap_int16_t(a, b) { int16_t swap = a; a = b; b = swap; }
#endif

/***********************************************************
 * Global variables
 ***********************************************************/
static LTDC_Layer_InitTypeDef LTDC_Layer_InitStruct;
static DMA2D_InitTypeDef DMA2D_InitStruct;
static DMA2D_FG_InitTypeDef DMA2D_FGInitStruct;
static DMA2D_BG_InitTypeDef DMA2D_BGInitStruct;
 /***********************************************************
 * Declaration internal functions
 ***********************************************************/
void DMA2D_Display_GPIOs_init ( void );

void DMA2D_InitAndTransfer( void );

void DMA2D_InitAndTransferNoWait( void );

void DMA2D_Display_IT_init(void);

 /***********************************************************
 * Function definitions
 ***********************************************************/
void DMA2D_Display_init(){
	SDRAM_FLASH_init();
	LTDC_InitTypeDef LTDC_InitStruct;
	// Activate clocks
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_LTDC, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_DMA2D, ENABLE );


	// Initialize GPIOs for display
	DMA2D_Display_GPIOs_init();
//	DMA_init();

	// Activate display clock
	//192 MHz / 3 = 64 MHz
	RCC_PLLSAIConfig( 192, 7, 3 );	//192, 7, 5
	//64 Mhz / 2 = 32 MHz
	RCC_LTDCCLKDivConfig(RCC_PLLSAIDivR_Div2);		//Sets the section PLLSAIDIVR (here: 2 -> f_LCD_CLK = f_PLLSAI_R / 2) in register RCC_DCKCFGR
	RCC_PLLSAICmd( ENABLE );
	while( RCC_GetFlagStatus( RCC_FLAG_PLLSAIRDY ) == RESET );


	// Initialization
	LTDC_InitStruct.LTDC_HSPolarity = HS_POLARITY;
	LTDC_InitStruct.LTDC_VSPolarity = VS_POLARITY;
	LTDC_InitStruct.LTDC_DEPolarity = DE_POLARITY;
	LTDC_InitStruct.LTDC_PCPolarity = PC_POLARITY;
	LTDC_InitStruct.LTDC_HorizontalSync = HORIZONTAL_SYNC;
	LTDC_InitStruct.LTDC_VerticalSync = VERTICAL_SYNC;
	LTDC_InitStruct.LTDC_AccumulatedHBP = ACCUMULATED_HBP;
	LTDC_InitStruct.LTDC_AccumulatedVBP = ACCUMULATED_VBP;
	LTDC_InitStruct.LTDC_AccumulatedActiveH = ACCUMULATED_ACTIVE_H;
	LTDC_InitStruct.LTDC_AccumulatedActiveW = ACCUMULATED_ACTIVE_W;
	LTDC_InitStruct.LTDC_TotalHeigh = TOTAL_HEIGH;
	LTDC_InitStruct.LTDC_TotalWidth = TOTAL_WIDTH;
	LTDC_InitStruct.LTDC_BackgroundBlueValue = 255;
	LTDC_InitStruct.LTDC_BackgroundGreenValue = 255;
	LTDC_InitStruct.LTDC_BackgroundRedValue = 255;
	LTDC_Init( &LTDC_InitStruct );

	LTDC_Layer_InitStruct.LTDC_HorizontalStart = WINDOW_X0;
	LTDC_Layer_InitStruct.LTDC_HorizontalStop = WINDOW_X1;
	LTDC_Layer_InitStruct.LTDC_VerticalStart = WINDOW_Y0;
	LTDC_Layer_InitStruct.LTDC_VerticalStop = WINDOW_Y1;
	LTDC_Layer_InitStruct.LTDC_PixelFormat = PIXEL_FORMAT;
	LTDC_Layer_InitStruct.LTDC_CFBStartAdress = LCD_START_ADDR;
	LTDC_Layer_InitStruct.LTDC_CFBLineLength = ( IMAGE_WIDTH * LCD_BPP ) + 3;
	LTDC_Layer_InitStruct.LTDC_CFBPitch = IMAGE_WIDTH * LCD_BPP;
	LTDC_Layer_InitStruct.LTDC_CFBLineNumber = IMAGE_HEIGHT;
	LTDC_Layer_InitStruct.LTDC_ConstantAlpha = 255;
	LTDC_Layer_InitStruct.LTDC_DefaultColorAlpha = 255;
	LTDC_Layer_InitStruct.LTDC_DefaultColorBlue = 0;
	LTDC_Layer_InitStruct.LTDC_DefaultColorGreen = 0;
	LTDC_Layer_InitStruct.LTDC_DefaultColorRed = 0;
	LTDC_Layer_InitStruct.LTDC_BlendingFactor_1 = LTDC_BlendingFactor1_PAxCA;
	LTDC_Layer_InitStruct.LTDC_BlendingFactor_2 = LTDC_BlendingFactor2_PAxCA;


	LTDC_LayerInit( LTDC_Layer1, &LTDC_Layer_InitStruct );

#ifdef LAYER_2_ACTIVE
	LTDC_Layer_InitStruct.LTDC_CFBStartAdress = LCD_START_ADDR + LCD_COUNT_BUFFER * LCD_BUFFER_SIZE;
	LTDC_LayerInit( LTDC_Layer2, &LTDC_Layer_InitStruct );
#endif

	LTDC_LayerCmd( LTDC_Layer1, ENABLE );

#ifdef LAYER_2_ACTIVE
	LTDC_LayerCmd( LTDC_Layer2, ENABLE );
#endif

	LTDC_ReloadConfig( LTDC_IMReload );
	LTDC_Cmd( ENABLE );
	LTDC_LayerAlpha( LTDC_Layer1, 255 );
#ifdef LAYER_2_ACTIVE
	LTDC_LayerAlpha(LTDC_Layer2, 255);
#endif
	LTDC_ReloadConfig( LTDC_VBReload );

	DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);
	DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_2);
#ifdef LAYER_2_ACTIVE
	DMA2D_Fill_Color(0x00FFFFFF, Layer_2, Buffer_1);
	DMA2D_Fill_Color(0x00FFFFFF, Layer_2, Buffer_2);
#endif
//	DMA2D_Display_IT_init();
}

void switch_buffer(Layer_e layer, Buffer_e buffer){
	if(layer == Layer_1){
		LTDC_Layer_InitStruct.LTDC_CFBStartAdress = LCD_START_ADDR + buffer * LCD_BUFFER_SIZE;
		LTDC_LayerInit( LTDC_Layer1, &LTDC_Layer_InitStruct);
	}
#ifdef LAYER_2_ACTIVE
	else if(layer == Layer_2){
		LTDC_Layer_InitStruct.LTDC_CFBStartAdress = LCD_START_ADDR + LCD_COUNT_BUFFER * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE;
		LTDC_LayerInit( LTDC_Layer2, &LTDC_Layer_InitStruct);
	}
#endif
	LTDC->SRCR = LTDC_SRCR_VBR;
}

/* Frame Buffer Switching AN4861
 * The LTDC provides different options to synchronize this workflow:
 *  Program the line interruption with the value of the last screen line, the interrupt handler
 * should flip the framebuffers and start the next framebuffer calculation.
 *  Program the shadow reload register (LTDC_SRCR) to Vertical blanking reload to
 * change the LTDC framebuffer address on the VSYNC period and Poll on VSYNC bit of
 * the LTDC_CDSR register to unblock the DMA2D.
 */
Buffer_e swap_buffer(Layer_e layer){


	static Buffer_e backbuffer = Buffer_1;

	// wait for the frame from the old buffer to be drawn completely (VSYNC=1)
	while((LTDC->CDSR & LTDC_CDSR_VSYNCS) == 0);


	switch_buffer(layer, backbuffer);

	// toggle the backbuffer. Next drawn will use a new backbuffer, while the TFT will be read from the buffer set in the line above
	if(backbuffer == Buffer_1) backbuffer = Buffer_2;
	else if(backbuffer == Buffer_2) backbuffer = Buffer_1;

	return backbuffer;
}

void set_layer_alpha(Layer_e layer, uint8_t alpha){
	if(layer == Layer_1) LTDC_LayerAlpha(LTDC_Layer1, alpha);
	else if(layer == Layer_2) LTDC_LayerAlpha(LTDC_Layer2, alpha);
	LTDC_ReloadConfig(LTDC_VBReload);
}

void DMA2D_DrawPixel (uint16_t xpos, uint16_t ypos, uint32_t color, Layer_e layer, Buffer_e buffer){
	if(xpos >= LCD_X_PIXEL || ypos >= LCD_Y_PIXEL) return;
	
	if(LCD_BPP == 4 || LCD_BPP == 3) 	*(__IO uint32_t *) (LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP) = color;
	else if(LCD_BPP == 2) 				*(__IO uint16_t *) (LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP) = (uint16_t) color;
}

void DMA2D_Fill_Color(uint32_t color, Layer_e layer, Buffer_e buffer){
	DMA2D_InitStruct.DMA2D_Mode = DMA2D_R2M;
	if(PIXEL_FORMAT == LTDC_Pixelformat_RGB565){
		uint16_t color16 = (uint16_t) color;
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB565;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x001F & color16);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x07E0 & color16) >> 5;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xF800 & color16) >> 11;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_RGB888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x0000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x00FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xFF0000 & color) >> 16;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_ARGB8888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_ARGB8888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x000000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x0000FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0x00FF0000 & color) >> 16;
		DMA2D_InitStruct.DMA2D_OutputAlpha =(0xFF000000 & color) >> 24;
	}
	DMA2D_InitStruct.DMA2D_OutputMemoryAdd = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE;
	DMA2D_InitStruct.DMA2D_OutputOffset = 0;
	DMA2D_InitStruct.DMA2D_NumberOfLine = LCD_Y_PIXEL;
	DMA2D_InitStruct.DMA2D_PixelPerLine = LCD_X_PIXEL;

	DMA2D_InitAndTransfer();
}

void DMA2D_Fill_Image(uint32_t addr_image, uint32_t source_format, Layer_e layer, Buffer_e buffer){
	DMA2D_StructInit(&DMA2D_InitStruct);
	DMA2D_InitStruct.DMA2D_Mode	= DMA2D_M2M_PFC;
	if(PIXEL_FORMAT == LTDC_Pixelformat_RGB565){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB565;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_RGB888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB888;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_ARGB8888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_ARGB8888;
	}

	DMA2D_InitStruct.DMA2D_NumberOfLine = LCD_Y_PIXEL;
	DMA2D_InitStruct.DMA2D_PixelPerLine = LCD_X_PIXEL;
	DMA2D_InitStruct.DMA2D_OutputOffset = 0;
	DMA2D_InitStruct.DMA2D_OutputMemoryAdd = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE;

	DMA2D_FG_StructInit(&DMA2D_FGInitStruct);
	DMA2D_FGInitStruct.DMA2D_FGCM = source_format;
	DMA2D_FGInitStruct.DMA2D_FGMA = addr_image;
	DMA2D_FGInitStruct.DMA2D_FGO = 0;

	DMA2D_InitAndTransfer();
}

void DMA2D_Draw_Image(uint16_t xpos, uint16_t ypos, uint16_t xsize, uint16_t ysize, uint8_t alpha, uint32_t alpha_mode, uint32_t addr_image, uint32_t source_format, Layer_e layer, Buffer_e buffer){
	if(xpos >= LCD_X_PIXEL || ypos >= LCD_Y_PIXEL) return;

	DMA2D_InitStruct.DMA2D_Mode	= DMA2D_M2M_BLEND;
	if(PIXEL_FORMAT == LTDC_Pixelformat_RGB565){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB565;
		DMA2D_BGInitStruct.DMA2D_BGCM = CM_RGB565;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_RGB888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB888;
		DMA2D_BGInitStruct.DMA2D_BGCM = CM_RGB888;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_ARGB8888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_ARGB8888;
		DMA2D_BGInitStruct.DMA2D_BGCM = CM_ARGB8888;
	}

	DMA2D_InitStruct.DMA2D_NumberOfLine = ysize;
	DMA2D_InitStruct.DMA2D_PixelPerLine = xsize;
	DMA2D_InitStruct.DMA2D_OutputOffset = LCD_X_PIXEL-xsize;
	DMA2D_InitStruct.DMA2D_OutputMemoryAdd = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP;

	DMA2D_FGInitStruct.DMA2D_FGCM = source_format;
	DMA2D_FGInitStruct.DMA2D_FGMA = addr_image;
	DMA2D_FGInitStruct.DMA2D_FGO = 0;
	//REPLACE_ALPHA_VALUE -> Replaces alpha value of image with parameter alpha;
	//COMBINE_ALPHA_VALUE -> Combines alpha value from image with parameter alpha;
	//NO_MODIF_ALPHA_VALUE -> Keeps alpha value from image, ignores parameter alpha;
	DMA2D_FGInitStruct.DMA2D_FGPFC_ALPHA_MODE = alpha_mode;
	DMA2D_FGInitStruct.DMA2D_FGPFC_ALPHA_VALUE = alpha;

	DMA2D_BGInitStruct.DMA2D_BGMA = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP;
	DMA2D_BGInitStruct.DMA2D_BGO = LCD_X_PIXEL-xsize;

	DMA2D_InitAndTransfer();
}

void DMA2D_Draw_X_Line(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t width, uint32_t color, Layer_e layer, Buffer_e buffer){
	if(xpos >= LCD_X_PIXEL || ypos >= LCD_Y_PIXEL) return;

	DMA2D_InitStruct.DMA2D_Mode = DMA2D_R2M;
	if(PIXEL_FORMAT == LTDC_Pixelformat_RGB565){
		uint16_t color16 = (uint16_t) color;
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB565;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x001F & color16);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x07E0 & color16) >> 5;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xF800 & color16) >> 11;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_RGB888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x0000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x00FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xFF0000 & color) >> 16;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_ARGB8888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_ARGB8888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x000000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x0000FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0x00FF0000 & color) >> 16;
		DMA2D_InitStruct.DMA2D_OutputAlpha =(0xFF000000 & color) >> 24;
	}
	ypos=ypos-((width-1)/2);
	DMA2D_InitStruct.DMA2D_OutputMemoryAdd = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP;
	DMA2D_InitStruct.DMA2D_OutputOffset = LCD_X_PIXEL-length;
	DMA2D_InitStruct.DMA2D_NumberOfLine = width;
	if(xpos + length > LCD_X_PIXEL) length = LCD_X_PIXEL - xpos;
	DMA2D_InitStruct.DMA2D_PixelPerLine = length;

	DMA2D_InitAndTransfer();
}

void DMA2D_Draw_Y_Line(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t width, uint32_t color, Layer_e layer, Buffer_e buffer){
	if(xpos >= LCD_X_PIXEL || ypos >= LCD_Y_PIXEL) return;

	DMA2D_InitStruct.DMA2D_Mode = DMA2D_R2M;
	if(PIXEL_FORMAT == LTDC_Pixelformat_RGB565){
		uint16_t color16 = (uint16_t) color;
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB565;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x001F & color16);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x07E0 & color16) >> 5;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xF800 & color16) >> 11;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_RGB888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x0000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x00FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xFF0000 & color) >> 16;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_ARGB8888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_ARGB8888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x000000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x0000FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0x00FF0000 & color) >> 16;
		DMA2D_InitStruct.DMA2D_OutputAlpha =(0xFF000000 & color) >> 24;
	}
	xpos=xpos-((width-1)/2);	// correct xpos to width
	DMA2D_InitStruct.DMA2D_OutputMemoryAdd = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP;
	DMA2D_InitStruct.DMA2D_OutputOffset = LCD_X_PIXEL-width;
	if(ypos + length > LCD_Y_PIXEL) length = LCD_Y_PIXEL - ypos;
	DMA2D_InitStruct.DMA2D_NumberOfLine = length;
	DMA2D_InitStruct.DMA2D_PixelPerLine =width;//1;

	DMA2D_InitAndTransfer();
}

void DMA2D_Draw_Line(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint32_t color, Layer_e layer, Buffer_e buffer)
{
  int16_t steep = abs(y1 - y0) > abs(x1 - x0);

  if (steep)
  {
    _swap_int16_t(x0, y0);
    _swap_int16_t(x1, y1);
  }
  if (x0 > x1)
  {
	_swap_int16_t(x0, x1);
	_swap_int16_t(y0, y1);
  }

  int16_t dx, dy;
  dx = x1 - x0;
  dy = abs(y1 - y0);
  int16_t err = dx / 2;
  int16_t ystep;

  if (y0 < y1)
	  ystep = 1;
  else
	  ystep = -1;

  for (; x0 <= x1; x0++)
  {
	  if (steep)
	  {
		  DMA2D_DrawPixel (y0, x0-1, color, layer, buffer);
		  DMA2D_DrawPixel (y0, x0,   color, layer, buffer);
		  DMA2D_DrawPixel (y0, x0+1, color, layer, buffer);
		  DMA2D_DrawPixel (y0-1, x0,   color, layer, buffer);
		  DMA2D_DrawPixel (y0+1, x0,   color, layer, buffer);
	  }
	   else
	   {
		   DMA2D_DrawPixel (x0, y0-1, color, layer, buffer);
		   DMA2D_DrawPixel (x0, y0,   color, layer, buffer);
		   DMA2D_DrawPixel (x0, y0+1, color, layer, buffer);
		   DMA2D_DrawPixel (x0-1, y0,   color, layer, buffer);
		   DMA2D_DrawPixel (x0+1, y0,   color, layer, buffer);
	   }
	   err -= dy;
	   if (err < 0)
	   {
	      y0 += ystep;
	      err += dx;
	    }
  }

}
void DMA2D_Draw_FilledRectangle(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t height, uint32_t color, Layer_e layer, Buffer_e buffer){
	if(xpos >= LCD_X_PIXEL || ypos >= LCD_Y_PIXEL)
		return;
	DMA2D_InitStruct.DMA2D_Mode = DMA2D_R2M;
	if(PIXEL_FORMAT == LTDC_Pixelformat_RGB565){
		uint16_t color16 = (uint16_t) color;
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB565;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x001F & color16);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x07E0 & color16) >> 5;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xF800 & color16) >> 11;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_RGB888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x0000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x00FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0xFF0000 & color) >> 16;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_ARGB8888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_ARGB8888;
		DMA2D_InitStruct.DMA2D_OutputBlue = (0x000000FF & color);
		DMA2D_InitStruct.DMA2D_OutputGreen =(0x0000FF00 & color) >> 8;
		DMA2D_InitStruct.DMA2D_OutputRed  = (0x00FF0000 & color) >> 16;
		DMA2D_InitStruct.DMA2D_OutputAlpha =(0xFF000000 & color) >> 24;
	}
	DMA2D_InitStruct.DMA2D_OutputMemoryAdd = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP;
	if(xpos + length > LCD_X_PIXEL) length = LCD_X_PIXEL - xpos;
	DMA2D_InitStruct.DMA2D_OutputOffset = LCD_X_PIXEL - length;
	if(ypos + height > LCD_Y_PIXEL) height = LCD_Y_PIXEL - height;
	DMA2D_InitStruct.DMA2D_NumberOfLine = height;
	DMA2D_InitStruct.DMA2D_PixelPerLine = length;

	DMA2D_InitAndTransfer();
}

void DMA2D_Draw_Rectangle(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t height, uint32_t color, uint8_t thickness, Layer_e layer, Buffer_e buffer){

	uint8_t i = thickness;
	while(i-- != 0)
	{
		DMA2D_Draw_X_Line(xpos , ypos + i, length+1,1, color, layer, buffer); // top line
		DMA2D_Draw_X_Line(xpos , ypos+height - i, length+1,1, color, layer, buffer); // bottom line

		DMA2D_Draw_Y_Line(xpos + i, ypos , height,1, color, layer, buffer); // left line
		DMA2D_Draw_Y_Line(xpos+length -i, ypos , height,1, color, layer, buffer); // right line
	}
}

uint16_t DMA2D_write_char(char sign, uint16_t xpos, uint16_t ypos, uint32_t color, tFont* font, Layer_e layer, Buffer_e buffer, Layer_e bglayer, Buffer_e bgbuffer){
	if(xpos >= LCD_X_PIXEL || ypos >= LCD_Y_PIXEL) return 0;
	uint8_t red5, red8, green6, green8, blue5, blue8;

	DMA2D_InitStruct.DMA2D_Mode	= DMA2D_M2M_BLEND;
	if(PIXEL_FORMAT == LTDC_Pixelformat_RGB565){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB565;
		DMA2D_BGInitStruct.DMA2D_BGCM = CM_RGB565;
		red5 = (color & 0xF800) >> 11;
		red8 = (red5 * 526 + 23) >> 6;
		green6 = (color & 0x07E0) >> 5;
		green8 = (green6 * 259 + 33) >> 6;
		blue5 = (color & 0x001F);
		blue8 = (blue5 * 526 + 23) >> 6;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_RGB888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_RGB888;
		DMA2D_BGInitStruct.DMA2D_BGCM = CM_RGB888;
		blue8 = color & 0x0000FF;
		green8 = (color & 0x00FF00) >> 8;
		red8  = (color & 0xFF0000) >> 16;
	}
	else if(PIXEL_FORMAT == LTDC_Pixelformat_ARGB8888){
		DMA2D_InitStruct.DMA2D_CMode = DMA2D_ARGB8888;
		DMA2D_BGInitStruct.DMA2D_BGCM = CM_ARGB8888;
		blue8 = color & 0x000000FF;
		green8 = (color & 0x0000FF00) >> 8;
		red8  = (color & 0x00FF0000) >> 16;
	}
	DMA2D_InitStruct.DMA2D_NumberOfLine = font->chars[sign - 0x20].image->height; //character height
	DMA2D_InitStruct.DMA2D_PixelPerLine = font->chars[sign - 0x20].image->width; //char width
	DMA2D_InitStruct.DMA2D_OutputOffset = LCD_X_PIXEL - font->chars[sign - 0x20].image->width;
	DMA2D_InitStruct.DMA2D_OutputMemoryAdd = LCD_START_ADDR + LCD_COUNT_BUFFER * layer * LCD_BUFFER_SIZE + buffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP;

	DMA2D_FG_StructInit(&DMA2D_FGInitStruct);
	DMA2D_FGInitStruct.DMA2D_FGMA = (uint32_t) font->chars[sign - 0x20].image->data;
	DMA2D_FGInitStruct.DMA2D_FGO = 0;
	DMA2D_FGInitStruct.DMA2D_FGCM =  CM_A8;
	DMA2D_FGInitStruct.DMA2D_FGC_BLUE = blue8;
	DMA2D_FGInitStruct.DMA2D_FGC_GREEN = green8;
	DMA2D_FGInitStruct.DMA2D_FGC_RED = red8;

	DMA2D_BG_StructInit(&DMA2D_BGInitStruct);
	DMA2D_BGInitStruct.DMA2D_BGMA = LCD_START_ADDR + LCD_COUNT_BUFFER * bglayer * LCD_BUFFER_SIZE + bgbuffer * LCD_BUFFER_SIZE + ypos * LCD_X_PIXEL * LCD_BPP + xpos * LCD_BPP;
	DMA2D_BGInitStruct.DMA2D_BGO = LCD_X_PIXEL - font->chars[sign - 0x20].image->width;

	DMA2D_InitAndTransfer();
	return font->chars[sign- 0x20].image->width;
}

uint16_t DMA2D_write_string(char* string, uint16_t xpos, uint16_t ypos, uint32_t color, tFont* font, Layer_e layer, Buffer_e buffer, Layer_e bglayer, Buffer_e bgbuffer){
	uint16_t x_offset = 0, x_add;
	while(*string != '\0'){
		x_add = DMA2D_write_char(*string, xpos + x_offset, ypos, color, font, layer, buffer, bglayer, bgbuffer);
		if(x_add < 0) return x_offset;
		x_offset += x_add;
		string++;
	}
	return x_offset;
}

uint16_t get_string_length(char* string, tFont* font){
	uint16_t length = 0;
	while(*string != '\0'){
		length += font->chars[*string - 0x20].image->width;
		string++;
	}
	return length;
}



 /***********************************************************
 * Definition internal functions
 ***********************************************************/
void DMA2D_Display_GPIOs_init ( void )
{
	GPIO_InitTypeDef GPIO_InitStruct;

	// Activate clocks
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOI, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOJ, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOK, ENABLE );

	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;

	// PI12 = LCD_HSYNC, PI13 = LCD_VSYNC, PI14 = LCD_CLK, PI15 = LCD_R0
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init( GPIOI, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOI, GPIO_PinSource12, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource13, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource14, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource15, GPIO_AF_LTDC );

	// PJ0  = LCD_R1, PJ1  = LCD_R2, PJ2  = LCD_R3, PJ3  = LCD_R4, PJ4  = LCD_R5,
	// PJ5  = LCD_R6, PJ6  = LCD_R7, PJ7  = LCD_G0, PJ8  = LCD_G1, PJ9  = LCD_G2,
	// PJ10 = LCD_G3, PJ11 = LCD_G4, PJ12 = LCD_B0, PJ13 = LCD_B1, PJ14 = LCD_B2,
	// PJ15 = LCD_B3
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 |
			GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 |
			GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 |
			GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 |
			GPIO_Pin_15;
	GPIO_Init( GPIOJ, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource0, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource1, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource2, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource3, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource4, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource5, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource6, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource7, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource8, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource9, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource10, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource11, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource12, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource13, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource14, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOJ, GPIO_PinSource15, GPIO_AF_LTDC );

	// PK0  = LCD_G5, PK1  = LCD_G6, PK2  = LCD_G7, PK3  = LCD_B4, PK4  = LCD_B5,
	// PK5  = LCD_B6, PK6  = LCD_B7, PK7  = LCD_DE
#if defined(DISPLAY_3_5_INCH)
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 |
			GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;

#elif defined(DISPLAY_5_INCH) || defined(DISPLAY_4_3_INCH)
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 |
			GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 |
			GPIO_Pin_7;
#endif
	/*GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 |
			GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 |
			GPIO_Pin_7;*/
	GPIO_Init( GPIOK, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOK, GPIO_PinSource0, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOK, GPIO_PinSource1, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOK, GPIO_PinSource2, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOK, GPIO_PinSource3, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOK, GPIO_PinSource4, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOK, GPIO_PinSource5, GPIO_AF_LTDC );
	GPIO_PinAFConfig( GPIOK, GPIO_PinSource6, GPIO_AF_LTDC );

#if defined(DISPLAY_5_INCH) || defined(DISPLAY_4_3_INCH)
	GPIO_PinAFConfig( GPIOK, GPIO_PinSource7, GPIO_AF_LTDC );
#endif
}

void DMA2D_InitAndTransfer( void ){
	DMA2D_DeInit();

	DMA2D_Init(&DMA2D_InitStruct);
	DMA2D_FGConfig(&DMA2D_FGInitStruct);
	DMA2D_BGConfig(&DMA2D_BGInitStruct);

	DMA2D_StartTransfer();

	//Wait until tranfer is finished
	while(DMA2D->CR & DMA2D_CR_START);
}

void DMA2D_InitAndTransferNoWait( void ){
	DMA2D_DeInit();

	DMA2D_Init(&DMA2D_InitStruct);
	DMA2D_FGConfig(&DMA2D_FGInitStruct);
	DMA2D_BGConfig(&DMA2D_BGInitStruct);

	DMA2D_StartTransfer();
}

void DMA2D_Display_IT_init( void ){
	NVIC_InitTypeDef NVIC_InitStruct;
	LTDC_LIPConfig(479);
	LTDC_ITConfig(LTDC_IT_LI, ENABLE);

	NVIC_InitStruct.NVIC_IRQChannel = LTDC_IRQn;
	NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
	NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 4;
	NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&NVIC_InitStruct);
}
