/***********************************************************
 *
 * d_Touch_FT5316.c
 * SYSTART GmbH
 * SAM, 22.06.2016
 *
 ***********************************************************/

/***********************************************************
 * Includes
 ***********************************************************/
#include "driver/d_Touch_FT5316.h"

#include "global_Display_Touch.h"
#include "driver/d_timer.h"

/***********************************************************
 * Declaration internal functions
 ***********************************************************/
void Touch_GPIOs_init ( void );
bool AreaCheck ( uint16_t LeftX, uint16_t LeftY, uint16_t RightX, uint16_t RightY );

/***********************************************************
 * Global variables
 ***********************************************************/
static TouchXY_t Touch;

/***********************************************************
 * Function definitions
 ***********************************************************/
void Touch_FT5316_init ( void )
{
	I2C_InitTypeDef I2C_InitStruct;

	Touch_GPIOs_init();

	// Activate clock
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_I2C1, ENABLE );

	// Initialize I2C
	I2C_InitStruct.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStruct.I2C_ClockSpeed = 100000;
	I2C_InitStruct.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStruct.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStruct.I2C_OwnAddress1 = 0x00;
	I2C_InitStruct.I2C_Ack = I2C_Ack_Enable;
	I2C_Init( I2C1, &I2C_InitStruct );
	I2C_Cmd( I2C1, ENABLE );

	Touch.xPos = 0;
	Touch.yPos = 0;
	Touch.event = RELEASE;
}

uint8_t Touch_FT5316_getNrTouches ( void )
{
	uint8_t countTouch = 0;
	uint8_t reg = 0x02;
//	tTimer timeout = timer_get_time();
//	tTimer current_time;

	while( I2C_GetFlagStatus( I2C1, I2C_FLAG_BUSY ) );

	// Start-Sequence
	I2C_GenerateSTART( I2C1, ENABLE );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_SB ) );

	// ACK disable
	I2C_AcknowledgeConfig(I2C1, DISABLE);

	// Slave-Address
	I2C_Send7bitAddress(I2C1, I2C_ADDRESS_FT5316, I2C_Direction_Transmitter );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_ADDR ) );

	// ADDR-Flag reset
	I2C1->SR2;

	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_TXE ) );

	// Send address
	I2C_SendData( I2C1, reg );

	while( ( !I2C_GetFlagStatus( I2C1, I2C_FLAG_TXE ) ) || ( !I2C_GetFlagStatus( I2C1, I2C_FLAG_BTF ) ) );

	// Start-Sequence
	I2C_GenerateSTART( I2C1, ENABLE );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_SB ) );

	// Slave-Address
	I2C_Send7bitAddress(I2C1, I2C_ADDRESS_FT5316, I2C_Direction_Receiver );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_ADDR ) );

	// Clear ADDR-Flag
	I2C1->SR2;

	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_RXNE ) );

	// Stop-Sequence
	I2C_GenerateSTOP(I2C1, ENABLE);

	// Read data
	countTouch = (int16_t)(I2C_ReceiveData( I2C1 ) );

	// ACK enable
	I2C_AcknowledgeConfig( I2C1, ENABLE );

	return countTouch;
}


TouchXY_t Touch_FT5316_getTouchXY ( void )
{
	//static TouchXY_t TouchXY;
	uint8_t data[4] = {0};
	uint8_t reg = 0x03, count = 0;
	uint32_t timer = I2C_TIMEOUT;

	// Start-Sequence
	I2C_GenerateSTART( I2C1, ENABLE );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_SB ) ) if(!timer--){Touch.event=0xFF; return Touch;}

	// ACK enable
	I2C_AcknowledgeConfig( I2C1, ENABLE );

	// Slave-Address
	I2C_Send7bitAddress(I2C1, I2C_ADDRESS_FT5316, I2C_Direction_Transmitter );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_ADDR ) ) if(!timer--) {Touch.event=0xFF; return Touch;}
	timer = I2C_TIMEOUT;

	// Clear ADDR-Flag
	I2C1->SR2;

	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_TXE ) ) if(!timer--) {Touch.event=0xFF; return Touch;}
	timer = I2C_TIMEOUT;

	// Send address
	I2C_SendData( I2C1, reg );

	while( ( !I2C_GetFlagStatus( I2C1, I2C_FLAG_TXE ) ) || ( !I2C_GetFlagStatus( I2C1, I2C_FLAG_BTF ) ) ) if(!timer--) {Touch.event=0xFF; return Touch;}
	timer = I2C_TIMEOUT;

	// Start-Sequence
	I2C_GenerateSTART( I2C1, ENABLE );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_SB ) ) if(!timer--) {Touch.event=0xFF; return Touch;}
	timer = I2C_TIMEOUT;

	// Slave-Address
	I2C_Send7bitAddress(I2C1, I2C_ADDRESS_FT5316, I2C_Direction_Receiver );
	while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_ADDR ) ) if(!timer--) {Touch.event=0xFF; return Touch;}
	timer = I2C_TIMEOUT;

	// Clear ADDR-Flag
	I2C1->SR2;

	for ( count = 0; count < 4; ++count )
	{
		if ( ( count + 1 ) >= 4 )
		{
			// ACK disable
			I2C_AcknowledgeConfig( I2C1, DISABLE );
			// Stop-Sequenz
			I2C_GenerateSTOP( I2C1, ENABLE );
		}

		while( !I2C_GetFlagStatus( I2C1, I2C_FLAG_RXNE ) ) if(!timer--) {Touch.event=0xFF; return Touch;}
		timer = I2C_TIMEOUT;

		data[count] = I2C_ReceiveData( I2C1 );
	}

	// ACK enable
	I2C_AcknowledgeConfig( I2C1, ENABLE );

	// Calculation of X- and Y-coordinates
	float xTemp, yTemp;
	//Get event type
	Touch.event = (data[0] >> 6);
	// Mask off Touch-ID and Event-Flag
	xTemp = ((data[0] << 8) + data[1]) & 0x0FFF;
	yTemp = ((data[2] << 8) + data[3]) & 0x0FFF;
#ifdef FT5426
#ifdef DISPLAY_3_5_INCH
	Touch.xPos = (uint16_t)((xTemp * 25 / 70) + 0.5);
	Touch.yPos = (uint16_t)((yTemp * 15 / 40) + 0.5);
#endif
#ifdef DISPLAY_4_3_INCH
	Touch.xPos = (uint16_t)((xTemp * 3 / 8) + 0.5);
	Touch.yPos = (uint16_t)((yTemp * 17 / 48) + 0.5);
#endif
#ifdef DISPLAY_5_INCH
	Touch.xPos = (uint16_t)((xTemp * 25 / 56) + 0.5);
	Touch.yPos = (uint16_t)((yTemp * 15 / 32) + 0.5);
#endif
#else
	TouchXY.xPos = xTemp;
	TouchXY.yPos = yTemp;
#endif

	return Touch;
}


/***********************************************************
 * Definitions internal functions
 ***********************************************************/
void Touch_GPIOs_init ( void )
{
	GPIO_InitTypeDef GPIO_InitStruct;
	EXTI_InitTypeDef EXTI_InitStruct;
	NVIC_InitTypeDef NVIC_InitStruct;
	uint32_t count = 0;

	// Activate clocks
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOB, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOD, ENABLE );
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_SYSCFG, ENABLE );
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_SYSCFG, ENABLE );

	// PB6 = I2C1_SCL, PB7 = I2C1_SDA
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed;
	GPIO_Init( GPIOB, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOB, GPIO_PinSource6, GPIO_AF_I2C1 );
	GPIO_PinAFConfig( GPIOB, GPIO_PinSource7, GPIO_AF_I2C1 );


	// Init PD3 as INT-Pin for Touch
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_3;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed;
	GPIO_Init( GPIOD, &GPIO_InitStruct );

	SYSCFG_EXTILineConfig( EXTI_PortSourceGPIOD, EXTI_PinSource3 );

	EXTI_InitStruct.EXTI_Line = EXTI_Line3;
	EXTI_InitStruct.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStruct.EXTI_Trigger = EXTI_Trigger_Falling;
	EXTI_InitStruct.EXTI_LineCmd = ENABLE;
	EXTI_Init( &EXTI_InitStruct );

	NVIC_InitStruct.NVIC_IRQChannel = EXTI3_IRQn;
	NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 4;
	NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init( &NVIC_InitStruct );

	// Init PA4 as RESET-Pin for LCD
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_4;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed;
	GPIO_Init( GPIOA, &GPIO_InitStruct );

	GPIO_ResetBits( GPIOA, GPIO_Pin_4 );
	for ( count = 0; count < 1800000; ++count );
	GPIO_SetBits( GPIOA, GPIO_Pin_4 );
	for ( count = 0; count < 900000; ++count );
}


void EXTI3_IRQHandler(void){
	if(EXTI_GetITStatus(EXTI_Line3) != RESET){
		//interrupt code
		EXTI_ClearITPendingBit(EXTI_Line3);
	}
}



bool AreaCheck ( uint16_t LeftX, uint16_t TopY, uint16_t RightX, uint16_t BottomY )
{
	if ( LeftX < Touch.xPos &&
		 TopY < Touch.yPos &&
		 RightX > Touch.xPos &&
		 BottomY > Touch.yPos )
	{
		return true;
	}
	return false;
}
