/**
  ******************************************************************************
  * @package TouchUART
  * @author  EBS-SYSTART GmbH
  * @version V1.0
  * @date    06-July-2021
  * @brief   Gets the coordinates of a touch input and sends them via USB
			 to a connected PC.
  ******************************************************************************
*/

/***********************************************************
 * Includes
 ***********************************************************/
#include <stdbool.h>
#include "stm32f4xx.h"
/* Driver */
#include "driver/d_uart.h"
#include "driver/d_timer.h"
#include "driver/d_Backlight.h"
#include "driver/d_Touch_FT5316.h"
#include "driver/d_Display_DMA2D.h"
 /* Applications */
#include "application/touch.h"

#if 0
#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
#endif /* For Debugging */


/*****************************************************************
 * Public variables
 ****************************************************************/
extern tImage ebssystart_logo_35;
extern tImage twoInone_display_logo;
extern tFont courier_new;

/***********************************************************
 * Private Function Prototypes
 ***********************************************************/
void HW_Init(void);
void display_init(void);


/*****************************************************************
 *
 * Function: int main(void)
 *
 * Summary: Entry point of display control application
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
int main(void) {
	/* Initialize the peripherals */
	HW_Init();
	Touch_FT5316_init();
	display_init();

	DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);
	DMA2D_write_string("Touch Me!", (HDP-9*16)/2, VDP/2-11, 0xFF000000, &courier_new, Layer_1, Buffer_1, Layer_1, Buffer_1);
	for(;;){
		process_touch_input();
	}
	return 0;
}

/***********************************************************
 * Private Functions
 ***********************************************************/

void WWDG_IRQHandler(){
	while(1){

	}
}
void TIM8_UP_TIM13_IRQHandler(){
	while(1){

	}
}
void LCD_TFT_IRQHandler(){
	while(1){

	}
}
void LCD_TFT_1_IRQHandler(){
	while(1){

	}
}

/*****************************************************************
 *
 * Function:void HW_Init(void)
 *
 * Summary: Various Hardware initializations
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
void HW_Init(void) {
	SystemInit();

	timer_init();
	uart_init();

	Backlight_init();
	Backlight_configPWM(95);
	Backlight_ON();
}


/*****************************************************************
 *
 * Function:  void display_init(void)
 *
 * Summary:   Initializes the display
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
void display_init(void) {

	SDRAM_FLASH_init();
	DMA2D_Display_init();

	switch_buffer(Layer_1, Buffer_1);
	DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);

	uint16_t image_width = ebssystart_logo_35.width;
	uint16_t image_height = ebssystart_logo_35.height;
	DMA2D_Draw_Image((HDP-image_width)/2, (VDP-image_height)/2, image_width, image_height, 0xFF, REPLACE_ALPHA_VALUE, (uint32_t)ebssystart_logo_35.data, CM_ARGB8888, Layer_1, Buffer_1);

	delay_ms(4000);
	image_width = twoInone_display_logo.width;
	image_height = twoInone_display_logo.height;
	DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);
	DMA2D_Draw_Image((HDP-image_width)/2, (VDP-image_height)/2, image_width, image_height, 0xFF, REPLACE_ALPHA_VALUE, (uint32_t)twoInone_display_logo.data, CM_ARGB8888, Layer_1, Buffer_1);

	delay_ms(4000);
}


#if 0 //For Debugging
PUTCHAR_PROTOTYPE
{
  USART_SendData(UART[0].UART, (uint8_t) ch);

  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(UART[0].UART, USART_FLAG_TC) == RESET)
  {}

  return ch;
}
#endif
